@@ -8,6 +8,9 @@
 # If the argument is an encrypted password it will ask the password and will
 # verify if it's valid.
 #
+# WARNING: don't use the character colon ':' as part of your name or password
+# since this character is used as field separator.
+#
 
 use strict;
 use warnings;
@@ -17,7 +20,9 @@ my $word;
 if(!$ARGV[0]) {
 	my $salt = join('', ('.', '/', 0..9, 'A'..'Z', 'a'..'z')[rand 64, rand 64]);
 	print "Password to encrypt: ";
+	system("stty -echo");
 	chomp($word = <STDIN>);
+	system("stty echo");
 	print crypt($word, $salt) . "\n";
 } else {
 	system("stty -echo");
