@@ -196,6 +196,7 @@ sub amdgpu_update {
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $amdgpu = $config->{amdgpu};
 	my $use_nan_for_missing_data = lc($amdgpu->{use_nan_for_missing_data} || "") eq "y" ? 1 : 0;
+	my $respect_power_state = lc($amdgpu->{respect_power_state} || "") eq "y" ? 1 : 0;
 
 	my @sensors;
 
@@ -216,21 +217,44 @@ sub amdgpu_update {
 				my $str = trim($gpu_group[$n] || "");
 				my @sensor_names = split(',', $amdgpu->{sensors}->{$str});
 
-				for(my $i_sensor = 0; $i_sensor < $number_of_values_per_gpu_in_rrd; $i_sensor++) {
-					if ($i_sensor < scalar(@sensor_names)) {
-						my $sensor_name = $sensor_names[$i_sensor];
-						chomp($sensor_name);
-						$sensor_name = trim($sensor_name);
-						if ($sensor_name ne "") {
-							my $sensor_file = $sensor_name;
-							if(open(IN, $sensor_file)) {
+				my $gpu_in_d3_state = 0;
+				if ($respect_power_state) {
+					my $power_state_sensor_name = $amdgpu->{power_states}->{$str};
+					if (defined($power_state_sensor_name)) {
+						chomp($power_state_sensor_name);
+						$power_state_sensor_name = trim($power_state_sensor_name);
+						if ($power_state_sensor_name ne "") {
+							my $power_state_sensor_file = $power_state_sensor_name;
+							if(open(IN, $power_state_sensor_file)) {
 								my $val = <IN>;
 								close(IN);
-								$val = trim($val);
-								chomp($val);
-								$sensors[$i_sensor] = $val;
+								if (index(lc($val), lc("D3")) != -1) {
+									$gpu_in_d3_state = 1;
+								}
 							} else {
-								logger("$myself: ERROR: unable to open '$sensor_file'.");
+								logger("$myself: ERROR: unable to open power state sensor file '$power_state_sensor_file'.");
+							}
+						}
+					}
+				}
+
+				if (!$gpu_in_d3_state) {
+					for(my $i_sensor = 0; $i_sensor < $number_of_values_per_gpu_in_rrd; $i_sensor++) {
+						if ($i_sensor < scalar(@sensor_names)) {
+							my $sensor_name = $sensor_names[$i_sensor];
+							chomp($sensor_name);
+							$sensor_name = trim($sensor_name);
+							if ($sensor_name ne "") {
+								my $sensor_file = $sensor_name;
+								if(open(IN, $sensor_file)) {
+									my $val = <IN>;
+									close(IN);
+									$val = trim($val);
+									chomp($val);
+									$sensors[$i_sensor] = $val;
+								} else {
+									logger("$myself: ERROR: unable to open '$sensor_file'.");
+								}
 							}
 						}
 					}
@@ -478,10 +502,10 @@ sub amdgpu_cgi {
 		"%4.2lf%s",
 		"%5.0lf%s",
 		"%5.0lf%s",
-		"%3.1lf%%",
-		"%3.1lf",
-		"%3.1lf",
-		"%3.1lf"
+		"%5.1lf%%",
+		"%5.1lf",
+		"%5.1lf",
+		"%5.1lf"
 	);
 
 	my @graphs_per_plot = (7, 8, 10, 9, [5, 6], 0, 1, 2, 3, 4); # To rearange the graphs
@@ -597,12 +621,12 @@ sub amdgpu_cgi {
 							push(@tmp, "COMMENT:      N/A\\n");
 						} else {
 							if($main_plots_with_average[$n_plot]) {
-								push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\: " . $legend_labels_per_sensor[$n_sensor]);
-								push(@tmp, "GPRINT:trans_" . $value_name . ":AVERAGE:  Average\\: " . $legend_labels_per_sensor[$n_sensor]);
-								push(@tmp, "GPRINT:trans_" . $value_name . ":MIN:  Min\\: " . $legend_labels_per_sensor[$n_sensor]);
-								push(@tmp, "GPRINT:trans_" . $value_name . ":MAX:  Max\\: " . $legend_labels_per_sensor[$n_sensor] . "\\n");
+								push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\:" . $legend_labels_per_sensor[$n_sensor]);
+								push(@tmp, "GPRINT:trans_" . $value_name . ":AVERAGE: Average\\:" . $legend_labels_per_sensor[$n_sensor]);
+								push(@tmp, "GPRINT:trans_" . $value_name . ":MIN: Min\\:" . $legend_labels_per_sensor[$n_sensor]);
+								push(@tmp, "GPRINT:trans_" . $value_name . ":MAX: Max\\:" . $legend_labels_per_sensor[$n_sensor] . "\\n");
 							} else {
-								push(@tmp, "GPRINT:trans_" . $value_name . ":LAST: Current\\: " . $legend_labels_per_sensor[$n_sensor] . "\\n");
+								push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\:" . $legend_labels_per_sensor[$n_sensor] . "\\n");
 							}
 						}
 					} else {
@@ -679,7 +703,7 @@ sub amdgpu_cgi {
 				$cdef_sensor_allvalues .= ",0,GT,1,UNKN,IF";
 			}
 			my $plot_title = $config->{graphs}->{'_amdgpu' . ($n_plot + 1)};
-			$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 3 + $n_plot]",
+			$pic = $rrd{$version}->("$IMG_DIR" . $IMG[$e * $number_of_sensor_values_in_use + $n_plot],
 				"--title=$plot_title ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
 				"--imgformat=$imgfmt_uc",
@@ -697,10 +721,10 @@ sub amdgpu_cgi {
 				@CDEF,
 				@tmp);
 			$err = RRDs::error;
-			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 3 + $n_plot]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . $IMG[$e * $number_of_sensor_values_in_use + $n_plot]. ": $err\n") if $err;
 			if(lc($config->{enable_zoom}) eq "y") {
 				($width, $height) = split('x', $config->{graph_size}->{zoom});
-				$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 3 + $n_plot]",
+				$picz = $rrd{$version}->("$IMG_DIR" . $IMGz[$e * $number_of_sensor_values_in_use + $n_plot],
 					"--title=$plot_title  ($tf->{nwhen}$tf->{twhen})",
 					"--start=-$tf->{nwhen}$tf->{twhen}",
 					"--imgformat=$imgfmt_uc",
@@ -719,13 +743,13 @@ sub amdgpu_cgi {
 					@CDEF,
 					@tmpz);
 				$err = RRDs::error;
-				push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 3 + $n_plot]: $err\n") if $err;
+				push(@output, "ERROR: while graphing $IMG_DIR" . $IMGz[$e * $number_of_sensor_values_in_use + $n_plot]. ": $err\n") if $err;
 			}
 			$e2 = $e + $n_plot + 1;
 			if($title || ($silent =~ /imagetag/ && $graph =~ /amdgpu$e2/)) {
 				if(lc($config->{enable_zoom}) eq "y") {
 					if(lc($config->{disable_javascript_void}) eq "y") {
-						push(@output, "      " . picz_a_element(config => $config, IMGz => $IMGz[$e * 3 + $n_plot], IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+						push(@output, "      " . picz_a_element(config => $config, IMGz => $IMGz[$e * $number_of_sensor_values_in_use + $n_plot], IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 					} else {
 						if($version eq "new") {
 							$picz_width = $picz->{image_width} * $config->{global_zoom};
@@ -734,10 +758,10 @@ sub amdgpu_cgi {
 							$picz_width = $width + 115;
 							$picz_height = $height + 100;
 						}
-						push(@output, "      " . picz_js_a_element(width => $picz_width, height => $picz_height, config => $config, IMGz => $IMGz[$e * 3 + $n_plot], IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+						push(@output, "      " . picz_js_a_element(width => $picz_width, height => $picz_height, config => $config, IMGz => $IMGz[$e * $number_of_sensor_values_in_use + $n_plot], IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 					}
 				} else {
-					push(@output, "      " . img_element(config => $config, IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+					push(@output, "      " . img_element(config => $config, IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 				}
 			}
 		}
@@ -748,11 +772,10 @@ sub amdgpu_cgi {
 
 			if($amdgpu->{desc}->{$k}) {
 				push(@output, "    <tr>\n");
-				push(@output, "      <td bgcolor='$colors->{title_bg_color}' colspan='2'>\n");
-				push(@output, "       <font face='Verdana, sans-serif' color='$colors->{title_fg_color}'>\n");
+				push(@output, "      <td class='td-title' colspan='2'>\n");
 				push(@output, "       <font size='-1'>\n");
-				push(@output, "        <b>&nbsp;&nbsp;$amdgpu->{desc}->{$k}<b>\n");
-				push(@output, "       </font></font>\n");
+				push(@output, "        <b>&nbsp;&nbsp;$amdgpu->{desc}->{$k}</b>\n");
+				push(@output, "       </font>\n");
 				push(@output, "      </td>\n");
 				push(@output, "    </tr>\n");
 			}
