@@ -53,6 +53,8 @@ sub du_init {
 		$heartbeat = 2 * $refresh_interval;
 	}
 
+	my @disk_list = split(',', $du->{list});
+
 	if(-e $rrd) {
 		$info = RRDs::info($rrd);
 		for my $key (keys %$info) {
@@ -76,8 +78,8 @@ sub du_init {
 				}
 			}
 		}
-		if(scalar(@ds) / 9 != scalar(my @fl = split(',', $du->{list}))) {
-			logger("$myself: Detected size mismatch between 'list' (" . scalar(my @fl = split(',', $du->{list})) . ") and $rrd (" . scalar(@ds) / 9 . "). Resizing it accordingly. All historical data will be lost. Backup file created.");
+		if(scalar(@ds) / 9 != scalar(@disk_list)) {
+			logger("$myself: Detected size mismatch between 'list' (" . scalar(@disk_list) . ") and $rrd (" . scalar(@ds) / 9 . "). Resizing it accordingly. All historical data will be lost. Backup file created.");
 			rename($rrd, "$rrd.bak");
 		}
 		if(scalar(@rra) < 12 + (4 * $config->{max_historic_years})) {
@@ -106,7 +108,7 @@ sub du_init {
 			push(@max, "RRA:MAX:0.5:1440:" . (365 * $n));
 			push(@last, "RRA:LAST:0.5:1440:" . (365 * $n));
 		}
-		for($n = 0; $n < scalar(my @fl = split(',', $du->{list})); $n++) {
+		for($n = 0; $n < scalar(@disk_list); $n++) {
 			push(@tmp, "DS:du" . $n . "_d1:GAUGE:" . $heartbeat . ":0:U");
 			push(@tmp, "DS:du" . $n . "_d2:GAUGE:" . $heartbeat . ":0:U");
 			push(@tmp, "DS:du" . $n . "_d3:GAUGE:" . $heartbeat . ":0:U");
@@ -162,6 +164,7 @@ sub du_update {
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $du = $config->{du};
 	my $args = $du->{extra_args} || "";
+	my $use_nan_for_missing_data = lc($du->{use_nan_for_missing_data} || "") eq "y" ? 1 : 0;
 
 	my @dirs;
 
@@ -177,8 +180,10 @@ sub du_update {
 		return if(($min + 60 * $hour) % int($refresh_interval / 60));
 	}
 
+	my @disk_list = split(',', $du->{list});
+
 	my $e = 0;
-	while($e < scalar(my @dl = split(',', $du->{list}))) {
+	while($e < scalar(@disk_list)) {
 		my $type;
 		my $e2 = 0;
 
@@ -190,7 +195,7 @@ sub du_update {
 		foreach my $i (split(',', $du->{desc}->{$e})) {
 			my $line;
 
-			$dirs[$e][$e2] = 0 unless defined $dirs[$e][$e2];
+			$dirs[$e][$e2] = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $dirs[$e][$e2];
 			$str = trim($i);
 			if(-d $str) {
 				if($type eq "size") {
@@ -215,9 +220,9 @@ sub du_update {
 	}
 
 	$e = 0;
-	while($e < scalar(my @dl = split(',', $du->{list}))) {
+	while($e < scalar(@disk_list)) {
 		for($n = 0; $n < 9; $n++) {
-			$dirs[$e][$n] = 0 unless defined $dirs[$e][$n];
+			$dirs[$e][$n] = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $dirs[$e][$n];
 			$rrdata .= ":" . $dirs[$e][$n];
 		}
 		$e++;
@@ -291,7 +296,8 @@ sub du_cgi {
 	}
 
 	$title = !$silent ? $title : "";
-
+	my $gap_on_all_nan = lc($du->{gap_on_all_nan} || "") eq "y" ? 1 : 0;
+	my @disk_list = split(',', $du->{list});
 
 	# text mode
 	#
@@ -312,7 +318,7 @@ sub du_cgi {
 		my $line3;
 		push(@output, "    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
 		push(@output, "    ");
-		for($n = 0; $n < scalar(my @dl = split(',', $du->{list})); $n++) {
+		for($n = 0; $n < scalar(@disk_list); $n++) {
 			$line1 = "";
 			foreach my $i (split(',', $du->{desc}->{$n})) {
 				$i = trim($i);
@@ -324,7 +330,7 @@ sub du_cgi {
 			}
 			if($line1) {
 				my $i = length($line1);
-				push(@output, sprintf(sprintf("%${i}s", sprintf("%s", trim($dl[$n])))));
+				push(@output, sprintf(sprintf("%${i}s", sprintf("%s", trim($disk_list[$n])))));
 			}
 		}
 		push(@output, "\n");
@@ -341,7 +347,7 @@ sub du_cgi {
 			$line = @$data[$n];
 			$time = $time - (1 / $tf->{ts});
 			push(@output, sprintf(" %2d$tf->{tc} ", $time));
-			for($n2 = 0; $n2 < scalar(my @dl = split(',', $du->{list})); $n2++) {
+			for($n2 = 0; $n2 < scalar(@disk_list); $n2++) {
 				$n3 = 0;
 				foreach my $i (split(',', $du->{desc}->{$n2})) {
 					$from = $n2 * 9 + $n3++;
@@ -375,7 +381,7 @@ sub du_cgi {
 		$u = "";
 	}
 
-	for($n = 0; $n < scalar(my @dl = split(',', $du->{list})); $n++) {
+	for($n = 0; $n < scalar(@disk_list); $n++) {
 		$str = $u . $package . $n . "." . $tf->{when} . ".$imgfmt_lc";
 		push(@IMG, $str);
 		unlink("$IMG_DIR" . $str);
@@ -386,21 +392,41 @@ sub du_cgi {
 		}
 	}
 
+	my $graphs_per_row = $du->{graphs_per_row};
+	my @linpad =(0) x scalar(@disk_list);
+	if ($graphs_per_row > 1) {
+		for(my $n = 0; $n < scalar(@disk_list); $n++) {
+			my @ls = split(',', $du->{desc}->{$n});
+			$linpad[$n] = scalar(@ls);
+		}
+		for(my $n = 0; $n < scalar(@linpad); $n++) {
+			if ($n % $graphs_per_row == 0) {
+				my $max_number_of_lines = 0;
+				for (my $sub_n = $n; $sub_n < min($n + $graphs_per_row, scalar(@linpad)); $sub_n++) {
+					$max_number_of_lines = max($max_number_of_lines, $linpad[$sub_n]);
+				}
+				for (my $sub_n = $n; $sub_n < min($n + $graphs_per_row, scalar(@linpad)); $sub_n++) {
+					$linpad[$sub_n] = $max_number_of_lines;
+				}
+			}
+		}
+	}
+
 	@riglim = @{setup_riglim($rigid[0], $limit[0])};
 	$n = 0;
-	while($n < scalar(my @dl = split(',', $du->{list}))) {
+	while($n < scalar(@disk_list)) {
 		if($title) {
 			if($n == 0) {
-				push(@output, main::graph_header($title, $du->{graphs_per_row}));
+				push(@output, main::graph_header($title, $graphs_per_row));
 			}
 			push(@output, "    <tr>\n");
 		}
-		for($n2 = 0; $n2 < $du->{graphs_per_row}; $n2++) {
+		for($n2 = 0; $n2 < $graphs_per_row; $n2++) {
 			my $type;
 			my @DEF0;
 			my @CDEF0;
 
-			last unless $n < scalar(my @dl = split(',', $du->{list}));
+			last unless $n < scalar(@disk_list);
 			if($title) {
 				push(@output, "    <td>\n");
 			}
@@ -422,7 +448,7 @@ sub du_cgi {
 				push(@DEF0, "DEF:d7=$rrd:du" . $n . "_d7:AVERAGE");
 				push(@DEF0, "DEF:d8=$rrd:du" . $n . "_d8:AVERAGE");
 				push(@DEF0, "DEF:d9=$rrd:du" . $n . "_d9:AVERAGE");
-				push(@CDEF0, "CDEF:allvalues=d1,d2,d3,d4,d5,d6,d7,d8,d9,+,+,+,+,+,+,+,+");
+				push(@CDEF0, ($gap_on_all_nan ? "CDEF:allvalues=d1,UN,0,1,IF,d2,UN,0,1,IF,d3,UN,0,1,IF,d4,UN,0,1,IF,d5,UN,0,1,IF,d6,UN,0,1,IF,d7,UN,0,1,IF,d8,UN,0,1,IF,d9,UN,0,1,IF,+,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=d1,d2,d3,d4,d5,d6,d7,d8,d9,+,+,+,+,+,+,+,+"));
 			# default type is 'bytes'
 			} else {
 				$type_label = "bytes";
@@ -435,7 +461,7 @@ sub du_cgi {
 				push(@DEF0, "DEF:dk7=$rrd:du" . $n . "_d7:AVERAGE");
 				push(@DEF0, "DEF:dk8=$rrd:du" . $n . "_d8:AVERAGE");
 				push(@DEF0, "DEF:dk9=$rrd:du" . $n . "_d9:AVERAGE");
-				push(@CDEF0, "CDEF:allvalues=dk1,dk2,dk3,dk4,dk5,dk6,dk7,dk8,dk9,+,+,+,+,+,+,+,+");
+				push(@CDEF0, ($gap_on_all_nan ? "CDEF:allvalues=dk1,UN,0,1,IF,dk2,UN,0,1,IF,dk3,UN,0,1,IF,dk4,UN,0,1,IF,dk5,UN,0,1,IF,dk6,UN,0,1,IF,dk7,UN,0,1,IF,dk8,UN,0,1,IF,dk9,UN,0,1,IF,+,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=dk1,dk2,dk3,dk4,dk5,dk6,dk7,dk8,dk9,+,+,+,+,+,+,+,+"));
 				push(@CDEF0, "CDEF:d1=dk1,1024,*");
 				push(@CDEF0, "CDEF:d2=dk2,1024,*");
 				push(@CDEF0, "CDEF:d3=dk3,1024,*");
@@ -460,7 +486,7 @@ sub du_cgi {
 				push(@tmpz, "LINE2:d" . ($e + 1) . $LC[$e] . ":$str");
 				$e++;
 			}
-			while($e < 9) {
+			while($e < $linpad[$n]) {
 				push(@tmp, "COMMENT: \\n");
 				$e++;
 			}
@@ -470,7 +496,7 @@ sub du_cgi {
 				push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
 			}
 			($width, $height) = split('x', $config->{graph_size}->{medium});
-			$str = substr(trim($dl[$n]), 0, 25);
+			$str = substr(trim($disk_list[$n]), 0, 25);
 			$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$n]",
 				"--title=$str  ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
