@@ -24,6 +24,7 @@ use strict;
 use warnings;
 use Monitorix;
 use RRDs;
+use Time::HiRes;
 use Cwd 'abs_path';
 use File::Basename;
 use Exporter 'import';
@@ -209,6 +210,409 @@ EOF
 	}
 }
 
+# This tries to find out the physical device name of each fs.
+sub get_device_name {
+	my ($myself, $config, $fs, $debug) = @_;
+
+	foreach my $k (sort keys %{$fs->{list}}) {
+		my @fsl = split(',', $fs->{list}->{$k});
+		my $d;
+		foreach my $f (@fsl) {
+			$d = "";
+			$f = trim($f);
+			$d = $fs->{devmap}->{$f} if $fs->{devmap}->{$f};
+			next unless !$d;
+
+			if($f ne "swap") {
+				my $pid;
+				eval {
+					local $SIG{'ALRM'} = sub {
+						if($pid) {
+							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
+							kill 9, $pid;
+						} else {
+							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
+						}
+					};
+					alarm($config->{timeout});
+					$pid = open(IN, "df -P '$f' |");
+					while(<IN>) {
+						if(/ $f$/) {
+							($d) = split(' ', $_);
+							last;
+						}
+					}
+					close(IN);
+					alarm(0);
+					chomp($d);
+				};
+			}
+
+			if($config->{os} eq "Linux" && $config->{kernel} gt "2.4") {
+				my $lvm;
+				my $lvm_disk;
+				my $is_md;
+				my $found;
+
+				if($f eq "swap") {
+					$d = `cat /proc/swaps | tail -1 | awk -F " " '{ print \$1 }'`;
+					chomp($d);
+				}
+
+				# check for device names using symbolic links
+				# e.g. /dev/disk/by-uuid/db312d12-0da6-44e5-a354-4c82118f4b66
+				if(-l $d) {
+					$d = abs_path(dirname($d) . "/" . readlink($d));
+					chomp($d);
+				}
+
+				# get the major and minor of $d
+				my $rdev = (stat($d))[6];
+				if(!$rdev) {
+					logger("$myself: Unable to detect the device name of '$f', I/O stats won't be shown in graph. If this is really a mount point then consider using <devmap> to map it manually to a device name.");
+					next;
+				}
+				my $minor = $rdev % 256;
+				my $major = int($rdev / 256);
+
+				# do exists in /proc/diskstats?
+				if($found = is_in_diskstats($d, $major, $minor)) {
+					$d = $found;
+					$fs->{devmap}->{$f} = $d;
+					logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
+					next;
+				}
+
+				logger("$myself: Unable to find major/minor in /proc/diskstats.") if $debug;
+
+				# check if device is using EVMS <http://evms.sourceforge.net/>
+				if($d =~ m/\/dev\/evms\//) {
+					$d = `evms_query disks $d`;
+					if($found = is_in_diskstats($d)) {
+						$d = $found;
+						$fs->{devmap}->{$f} = $d;
+						logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
+						next;
+					}
+				}
+
+				$d =~ s/^.*dev\///;	# remove the /dev/ prefix
+				$d =~ s/^.*mapper\///;	# remove the mapper/ prefix
+
+				# check if the device is under a crypt LUKS (encrypted fs)
+				my $dev;
+				if($dev = is_luks($d)) {
+					$d = $dev;
+				}
+
+				# do exists in /proc/diskstats?
+				if($found = is_in_diskstats($d)) {
+					$d = $found;
+					$fs->{devmap}->{$f} = $d;
+					logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
+					next;
+				}
+
+				# check if the device is in a LVM
+				$lvm = $d;
+				$lvm =~ s/-.*//;
+				if($lvm ne $d) {	# probably LVM
+					if(system("pvs >/dev/null 2>&1") == 0 && $lvm) {
+						$lvm_disk = `pvs --noheadings | grep $lvm | tail -1 | awk -F " " '{ print \$1 }'`;
+						chomp($lvm_disk);
+						$lvm_disk =~ s/^.*dev\///;	# remove the /dev/ prefix
+						$lvm_disk =~ s/^.*mapper\///;	# remove the mapper/ prefix
+						if(!($lvm_disk =~ m/md/)) {
+							if($lvm_disk =~ m/cciss/) {
+								# LVM over a CCISS disk (/dev/cciss/c0d0)
+								$d = $lvm_disk;
+								chomp($d);
+							} elsif($dev = is_luks($lvm_disk)) {
+								$d = $dev;
+							} else {
+								# LVM over a direct disk (/dev/sda1)
+								$d = $lvm_disk;
+								chomp($d);
+							}
+						} else {
+							# LVM over Linux RAID combination (/dev/md1)
+							$d = $lvm_disk;
+							chomp($d);
+						}
+					}
+				}
+			} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
+				if($f eq "swap") {
+					if($config->{os} eq "FreeBSD" || $config->{os} eq "NetBSD") {
+						$d = `swapinfo | tail -1 | awk -F " " '{ print \$1 }'`;
+						chomp($d);
+					}
+					if($config->{os} eq "OpenBSD") {
+						$d = `swapctl -l | tail -1 | awk -F " " '{ print \$1 }'`;
+						chomp($d);
+					}
+				}
+
+				# remove the /dev/ prefix
+				if ($d =~ s/^.*dev\///) {
+					# not ZFS; get the device name, eg ada0; md0; ad10
+					$d =~ s/^(\D+\d*)\D.*/$1/;
+				} else {
+					# Just take ZFS pool name
+					$d =~ s,^([^/]*)/.*,$1,;
+				}
+			}
+			$fs->{devmap}->{$f} = $d;
+			logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
+		}
+	}
+}
+
+sub measure {
+	my ($myself, $config, $fs) = @_;
+	my $use_nan_for_missing_data = lc($fs->{use_nan_for_missing_data} || "") eq "y" ? 1 : 0;
+
+	my $val;
+	my $str;
+
+	my $n;
+	my $rrdata = "N";
+
+	my $e = 0;
+	foreach my $k (sort keys %{$fs->{list}}) {
+		my @fsl = split(',', $fs->{list}->{$k});
+		for($n = 0; $n < 8; $n++) {
+			my $use;
+			my $ioa;
+			my $tim;
+			my $ino;
+
+			my $used;
+			my $free;
+
+			my @tmp;
+
+			my $f = trim($fsl[$n]) || "";
+			if($f && $f eq "swap") {
+				if($config->{os} eq "Linux") {
+					open(IN, "free |");
+					while(<IN>) {
+						if(/^Swap:\s+\d+\s+(\d+)\s+(\d+)\s*$/) {
+							$used = $1;
+							$free = $2;
+						}
+					}
+					close(IN);
+				} elsif($config->{os} eq "FreeBSD") {
+					open(IN, "swapinfo -k |");
+					while(<IN>) {
+						if(/^.*?\s+\d+\s+(\d+)\s+(\d+)\s+\d+\%$/) {
+							$used = $1;
+							$free = $2;
+						}
+					}
+					close(IN);
+				} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
+					open(IN, "pstat -sk |");
+					while(<IN>) {
+						if(/^swap_device\s+\d+\s+(\d+)\s+(\d+) /) {
+							$used = $1;
+							$free = $2;
+						}
+					}
+					close(IN);
+				}
+
+				if(defined($used) && defined($free)) {
+					chomp($used, $free);
+					# prevents a division by 0 if swap device is not used
+					$use = ($used * 100) / ($used + $free) unless $used + $free == 0;
+				}
+			} elsif($f) {
+				my $pid;
+				eval {
+					local $SIG{'ALRM'} = sub {
+						if($pid) {
+							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
+							kill 9, $pid;
+						} else {
+							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
+						}
+					};
+					alarm($config->{timeout});
+					$pid = open(IN, "df -P '$f' |");
+					while(<IN>) {
+						if(/ $f$/) {
+							@tmp = split(' ', $_);
+							last;
+						}
+					}
+					close(IN);
+					alarm(0);
+				};
+				(undef, undef, $used, $free) = @tmp;
+				if(defined($used) && defined($free)) {
+					chomp($used, $free);
+					# prevents a division by 0 if device is not responding
+					$use = ($used * 100) / ($used + $free) unless $used + $free == 0;
+				}
+
+				eval {
+					local $SIG{'ALRM'} = sub {
+						if($pid) {
+							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
+							kill 9, $pid;
+						} else {
+							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
+						}
+					};
+					alarm($config->{timeout});
+					if($config->{os} eq "Linux") {
+						$pid = open(IN, "df -P -i '$f' |");
+					} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD") {
+						$pid = open(IN, "df -i '$f' |");
+					}
+					while(<IN>) {
+						if(/ $f$/) {
+							@tmp = split(' ', $_);
+							last;
+						}
+					}
+					close(IN);
+					alarm(0);
+				};
+				if($config->{os} eq "Linux") {
+					(undef, undef, $used, $free) = @tmp;
+				} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD") {
+					(undef, undef, undef, undef, undef, $used, $free) = @tmp;
+				}
+				if(defined($used) && defined($free)) {
+					chomp($used, $free);
+					# prevents a division by 0 if device is not responding
+					$ino = ($used * 100) / ($used + $free) unless $used + $free == 0;
+				}
+
+				# check alerts for each filesystem
+				my @al = split(',', $fs->{alerts}->{$f} || "");
+				if(scalar(@al)) {
+					my $timeintvl = trim($al[0]);
+					my $threshold = trim($al[1]);
+					my $script = trim($al[2]);
+
+					if(!$threshold || $use < $threshold) {
+						$config->{fs_hist}->{$f} = 0;
+					} else {
+						if(!$config->{fs_hist}->{$f}) {
+							$config->{fs_hist}->{$f} = time;
+						}
+						if($config->{fs_hist}->{$f} > 0 && (time - $config->{fs_hist}->{$f}) >= $timeintvl) {
+							if(-x $script) {
+								logger("$myself: alert on filesystem '$f': executing script '$script'.");
+								system($script . " " . $timeintvl . " " . $threshold . " " . $use);
+							} else {
+								logger("$myself: ERROR: script '$script' doesn't exist or don't has execution permissions.");
+							}
+							$config->{fs_hist}->{$f} = time;
+						}
+					}
+				}
+			}
+
+			my $read_cnt;
+			my $read_sec;
+			my $write_cnt;
+			my $write_sec;
+			my $d = $fs->{devmap}->{$f};
+			if($d) {
+				if($config->{os} eq "Linux") {
+					if($config->{kernel} gt "2.4") {
+						if(open(IN, "/proc/diskstats")) {
+							while(<IN>) {
+								if(/ $d /) {
+									@tmp = split(' ', $_);
+									last;
+								}
+							}
+							close(IN);
+						}
+						(undef, undef, undef, $read_cnt, undef, undef, $read_sec, $write_cnt, undef, undef, $write_sec) = @tmp;
+					} else {
+						my $io;
+						open(IN, "/proc/stat");
+						while(<IN>) {
+							if(/^disk_io/) {
+								(undef, undef, $io) = split(':', $_);
+								last;
+							}
+						}
+						close(IN);
+						(undef, $read_cnt, $read_sec, $write_cnt, $write_sec) = split(',', $io);
+						$write_sec =~ s/\).*$//;
+					}
+				} elsif($config->{os} eq "FreeBSD") {
+					@tmp = split(' ', `iostat -xI '$d' | grep -w '$d'`);
+					if(@tmp) {
+						(undef, $read_cnt, $write_cnt, $read_sec, $write_sec) = @tmp;
+						$read_cnt = int($read_cnt);
+						$write_cnt = int($write_cnt);
+						$read_sec = int($read_sec);
+						$write_sec = int($write_sec);
+					} else {
+						@tmp = split(' ', `iostat -dI | tail -1`);
+						(undef, $read_cnt, $read_sec) = @tmp;
+						$write_cnt = 0;
+						$write_sec = 0;
+						chomp($read_sec);
+						$read_sec = int($read_sec);
+					}
+				} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
+					@tmp = split(' ', `iostat -DI | tail -1`);
+					($read_cnt, $read_sec) = @tmp;
+					$write_cnt = 0;
+					$write_sec = 0;
+					chomp($read_sec);
+					$read_sec = int($read_sec);
+				}
+			}
+
+			my $epoc_identifier = "last_epoc_" . $e . "_" . $n;
+			my $last_epoc = ($config->{fs_hist}->{$epoc_identifier} || 0);
+			my $epoc = Time::HiRes::time();
+			$config->{fs_hist}->{$epoc_identifier} = $epoc;
+			my $delta_t = ($last_epoc ne 0) ? ($epoc - $last_epoc) : 60;
+
+			if(defined($read_cnt) && defined($write_cnt) &&
+			   defined($read_sec) && defined($write_sec)) {
+				$ioa = ($read_cnt || 0) + ($write_cnt || 0);
+				$tim = ($read_sec || 0) + ($write_sec || 0);
+
+				$str = $e . "_ioa" . $n;
+				$val = $ioa;
+				$ioa = $val - ($config->{fs_hist}->{$str} || 0);
+				$ioa = 0 unless $val != $ioa;
+				$ioa /= $delta_t;
+				$config->{fs_hist}->{$str} = $val;
+
+				$str = $e . "_tim" . $n;
+				$val = $tim;
+				$tim = $val - ($config->{fs_hist}->{$str} || 0);
+				$tim = 0 unless $val != $tim;
+				$tim /= $delta_t;
+				$config->{fs_hist}->{$str} = $val;
+			}
+
+			$use = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $use;
+			$ioa = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $ioa;
+			$tim = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $tim;
+			$ino = ($use_nan_for_missing_data ? (0+"nan") : 0) unless defined $ino;
+
+			$rrdata .= ":$use:$ioa:$tim:$ino:NaN:NaN:NaN:NaN";
+		}
+		$e++;
+	}
+	return $rrdata;
+}
+
 sub fs_init {
 	my $myself = (caller(0))[3];
 	my ($package, $config, $debug) = @_;
@@ -373,160 +777,13 @@ sub fs_init {
 		}
 	}
 
-	# This tries to find out the physical device name of each fs.
-	foreach my $k (sort keys %{$fs->{list}}) {
-		my @fsl = split(',', $fs->{list}->{$k});
-		my $d;
-		foreach my $f (@fsl) {
-			$d = "";
-			$f = trim($f);
-			$d = $fs->{devmap}->{$f} if $fs->{devmap}->{$f};
-			next unless !$d;
-
-			if($f ne "swap") {
-				my $pid;
-				eval {
-					local $SIG{'ALRM'} = sub {
-						if($pid) {
-							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
-							kill 9, $pid;
-						} else {
-							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
-						}
-					};
-					alarm($config->{timeout});
-					$pid = open(IN, "df -P '$f' |");
-					while(<IN>) {
-						if(/ $f$/) {
-							($d) = split(' ', $_);
-							last;
-						}
-					}
-					close(IN);
-					alarm(0);
-					chomp($d);
-				};
-			}
-
-			if($config->{os} eq "Linux" && $config->{kernel} gt "2.4") {
-				my $lvm;
-				my $lvm_disk;
-				my $is_md;
-				my $found;
-
-				if($f eq "swap") {
-					$d = `cat /proc/swaps | tail -1 | awk -F " " '{ print \$1 }'`;
-					chomp($d);
-				}
-
-				# check for device names using symbolic links
-				# e.g. /dev/disk/by-uuid/db312d12-0da6-44e5-a354-4c82118f4b66
-				if(-l $d) {
-					$d = abs_path(dirname($d) . "/" . readlink($d));
-					chomp($d);
-				}
-
-				# get the major and minor of $d
-				my $rdev = (stat($d))[6];
-				if(!$rdev) {
-					logger("$myself: Unable to detect the device name of '$f', I/O stats won't be shown in graph. If this is really a mount point then consider using <devmap> to map it manually to a device name.");
-					next;
-				}
-				my $minor = $rdev % 256;
-				my $major = int($rdev / 256);
-
-				# do exists in /proc/diskstats?
-				if($found = is_in_diskstats($d, $major, $minor)) {
-					$d = $found;
-					$fs->{devmap}->{$f} = $d;
-					logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
-					next;
-				}
-
-				logger("$myself: Unable to find major/minor in /proc/diskstats.") if $debug;
-
-				# check if device is using EVMS <http://evms.sourceforge.net/>
-				if($d =~ m/\/dev\/evms\//) {
-					$d = `evms_query disks $d`;
-					if($found = is_in_diskstats($d)) {
-						$d = $found;
-						$fs->{devmap}->{$f} = $d;
-						logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
-						next;
-					}
-				}
-
-				$d =~ s/^.*dev\///;	# remove the /dev/ prefix
-				$d =~ s/^.*mapper\///;	# remove the mapper/ prefix
-
-				# check if the device is under a crypt LUKS (encrypted fs)
-				my $dev;
-				if($dev = is_luks($d)) {
-					$d = $dev;
-				}
-
-				# do exists in /proc/diskstats?
-				if($found = is_in_diskstats($d)) {
-					$d = $found;
-					$fs->{devmap}->{$f} = $d;
-					logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
-					next;
-				}
-
-				# check if the device is in a LVM
-				$lvm = $d;
-				$lvm =~ s/-.*//;
-				if($lvm ne $d) {	# probably LVM
-					if(system("pvs >/dev/null 2>&1") == 0 && $lvm) {
-						$lvm_disk = `pvs --noheadings | grep $lvm | tail -1 | awk -F " " '{ print \$1 }'`;
-						chomp($lvm_disk);
-						$lvm_disk =~ s/^.*dev\///;	# remove the /dev/ prefix
-						$lvm_disk =~ s/^.*mapper\///;	# remove the mapper/ prefix
-						if(!($lvm_disk =~ m/md/)) {
-							if($lvm_disk =~ m/cciss/) {
-								# LVM over a CCISS disk (/dev/cciss/c0d0)
-								$d = $lvm_disk;
-								chomp($d);
-							} elsif($dev = is_luks($lvm_disk)) {
-								$d = $dev;
-							} else {
-								# LVM over a direct disk (/dev/sda1)
-								$d = $lvm_disk;
-								chomp($d);
-							}
-						} else {
-							# LVM over Linux RAID combination (/dev/md1)
-							$d = $lvm_disk;
-							chomp($d);
-						}
-					}
-				}
-			} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
-				if($f eq "swap") {
-					if($config->{os} eq "FreeBSD" || $config->{os} eq "NetBSD") {
-						$d = `swapinfo | tail -1 | awk -F " " '{ print \$1 }'`;
-						chomp($d);
-					}
-					if($config->{os} eq "OpenBSD") {
-						$d = `swapctl -l | tail -1 | awk -F " " '{ print \$1 }'`;
-						chomp($d);
-					}
-				}
-
-				# remove the /dev/ prefix
-				if ($d =~ s/^.*dev\///) {
-					# not ZFS; get the device name, eg ada0; md0; ad10
-					$d =~ s/^(\D+\d*)\D.*/$1/;
-				} else {
-					# Just take ZFS pool name
-					$d =~ s,^([^/]*)/.*,$1,;
-				}
-			}
-			$fs->{devmap}->{$f} = $d;
-			logger("$myself: Detected physical device name for $f in '$d'.") if $debug;
-		}
+	# save the original values in <devmap>
+	foreach my $d (sort keys %{$fs->{devmap}}) {
+		$config->{fs_orig_devmap}->{$d} = $fs->{devmap}->{$d};
 	}
 
+	get_device_name($myself, $config, $fs, $debug);
+
 	# check for deprecated options
 	if($fs->{alerts}->{rootfs_enabled} || $fs->{alerts}->{rootfs_timeintvl} || $fs->{alerts}->{rootfs_threshold} || $fs->{alerts}->{rootfs_script}) {
 		logger("$myself: WARNING: you have deprecated options in the <alerts> section. Please read the monitorix.conf(5) man page and consider also upgrade your current configuration file.");
@@ -534,6 +791,7 @@ sub fs_init {
 
 	$config->{fs_hist} = ();
 	push(@{$config->{func_update}}, $package);
+	measure($myself, $config, $fs); # Call to measuring routine to initialize the last values for calculating the differences. This way, the first update call will actually measure correct values.
 	logger("$myself: Ok") if $debug;
 }
 
@@ -571,227 +829,18 @@ sub fs_update {
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $fs = $config->{fs};
 
-	my @tmp;
-	my $val;
-	my $str;
+	if(lc($fs->{has_removable_devices} || "") eq "y") {
+		undef($config->{fs}->{devmap});
 
-	my $n;
-	my $rrdata = "N";
-
-	my $e = 0;
-	foreach my $k (sort keys %{$fs->{list}}) {
-		my @fsl = split(',', $fs->{list}->{$k});
-		for($n = 0; $n < 8; $n++) {
-			my $use = 0;
-			my $ioa = 0;
-			my $tim = 0;
-			my $ino = 0;
-
-			my $used = 0;
-			my $free = 0;
-
-			my $f = trim($fsl[$n]) || "";
-			if($f && $f eq "swap") {
-				if($config->{os} eq "Linux") {
-					open(IN, "free |");
-					while(<IN>) {
-						if(/^Swap:\s+\d+\s+(\d+)\s+(\d+)\s*$/) {
-							$used = $1;
-							$free = $2;
-						}
-					}
-					close(IN);
-				} elsif($config->{os} eq "FreeBSD") {
-					open(IN, "swapinfo -k |");
-					while(<IN>) {
-						if(/^.*?\s+\d+\s+(\d+)\s+(\d+)\s+\d+\%$/) {
-							$used = $1;
-							$free = $2;
-						}
-					}
-					close(IN);
-				} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
-					open(IN, "pstat -sk |");
-					while(<IN>) {
-						if(/^swap_device\s+\d+\s+(\d+)\s+(\d+) /) {
-							$used = $1;
-							$free = $2;
-						}
-					}
-					close(IN);
-				}
-
-				chomp($used, $free);
-				# prevents a division by 0 if swap device is not used
-				$use = ($used * 100) / ($used + $free) unless $used + $free == 0;
-			} elsif($f) {
-				my $pid;
-				@tmp = (0) x 10;
-				eval {
-					local $SIG{'ALRM'} = sub {
-						if($pid) {
-							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
-							kill 9, $pid;
-						} else {
-							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
-						}
-						@tmp = (0, 0, 0, 0);
-					};
-					alarm($config->{timeout});
-					$pid = open(IN, "df -P '$f' |");
-					while(<IN>) {
-						if(/ $f$/) {
-							@tmp = split(' ', $_);
-							last;
-						}
-					}
-					close(IN);
-					alarm(0);
-				};
-				(undef, undef, $used, $free) = @tmp;
-				chomp($used, $free);
-				# prevents a division by 0 if device is not responding
-				$use = ($used * 100) / ($used + $free) unless $used + $free == 0;
-
-				eval {
-					local $SIG{'ALRM'} = sub {
-						if($pid) {
-							logger("$myself: Timeout! Process with PID '$pid' still hung after $config->{timeout} secs. Killed.");
-							kill 9, $pid;
-						} else {
-							logger("$myself: WARNING: \$pid has no value ('$pid') in ALRM sighandler.");
-						}
-						@tmp = (0, 0, 0, 0, 0, 0, 0);
-					};
-					alarm($config->{timeout});
-					if($config->{os} eq "Linux") {
-						$pid = open(IN, "df -P -i '$f' |");
-					} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD") {
-						$pid = open(IN, "df -i '$f' |");
-					}
-					while(<IN>) {
-						if(/ $f$/) {
-							@tmp = split(' ', $_);
-							last;
-						}
-					}
-					close(IN);
-					alarm(0);
-				};
-				if($config->{os} eq "Linux") {
-					(undef, undef, $used, $free) = @tmp;
-				} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD") {
-					(undef, undef, undef, undef, undef, $used, $free) = @tmp;
-				}
-				chomp($used, $free);
-				# prevents a division by 0 if device is not responding
-				$ino = ($used * 100) / ($used + $free) unless $used + $free == 0;
-
-				# check alerts for each filesystem
-				my @al = split(',', $fs->{alerts}->{$f} || "");
-				if(scalar(@al)) {
-					my $timeintvl = trim($al[0]);
-					my $threshold = trim($al[1]);
-					my $script = trim($al[2]);
-
-					if(!$threshold || $use < $threshold) {
-						$config->{fs_hist}->{$f} = 0;
-					} else {
-						if(!$config->{fs_hist}->{$f}) {
-							$config->{fs_hist}->{$f} = time;
-						}
-						if($config->{fs_hist}->{$f} > 0 && (time - $config->{fs_hist}->{$f}) >= $timeintvl) {
-							if(-x $script) {
-								logger("$myself: alert on filesystem '$f': executing script '$script'.");
-								system($script . " " . $timeintvl . " " . $threshold . " " . $use);
-							} else {
-								logger("$myself: ERROR: script '$script' doesn't exist or don't has execution permissions.");
-							}
-							$config->{fs_hist}->{$f} = time;
-						}
-					}
-				}
-			}
-
-			my $read_cnt = 0;
-			my $read_sec = 0;
-			my $write_cnt = 0;
-			my $write_sec = 0;
-			my $d = $fs->{devmap}->{$f};
-			if($d) {
-				if($config->{os} eq "Linux") {
-					if($config->{kernel} gt "2.4") {
-						if(open(IN, "/proc/diskstats")) {
-							while(<IN>) {
-								if(/ $d /) {
-									@tmp = split(' ', $_);
-									last;
-								}
-							}
-							close(IN);
-						}
-						(undef, undef, undef, $read_cnt, undef, undef, $read_sec, $write_cnt, undef, undef, $write_sec) = @tmp;
-					} else {
-						my $io;
-						open(IN, "/proc/stat");
-						while(<IN>) {
-							if(/^disk_io/) {
-								(undef, undef, $io) = split(':', $_);
-								last;
-							}
-						}
-						close(IN);
-						(undef, $read_cnt, $read_sec, $write_cnt, $write_sec) = split(',', $io);
-						$write_sec =~ s/\).*$//;
-					}
-				} elsif($config->{os} eq "FreeBSD") {
-					@tmp = split(' ', `iostat -xI '$d' | grep -w '$d'`);
-					if(@tmp) {
-						(undef, $read_cnt, $write_cnt, $read_sec, $write_sec) = @tmp;
-						$read_cnt = int($read_cnt);
-						$write_cnt = int($write_cnt);
-						$read_sec = int($read_sec);
-						$write_sec = int($write_sec);
-					} else {
-						@tmp = split(' ', `iostat -dI | tail -1`);
-						(undef, $read_cnt, $read_sec) = @tmp;
-						$write_cnt = 0;
-						$write_sec = 0;
-						chomp($read_sec);
-						$read_sec = int($read_sec);
-					}
-				} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
-					@tmp = split(' ', `iostat -DI | tail -1`);
-					($read_cnt, $read_sec) = @tmp;
-					$write_cnt = 0;
-					$write_sec = 0;
-					chomp($read_sec);
-					$read_sec = int($read_sec);
-				}
-			}
-
-			$ioa = ($read_cnt || 0) + ($write_cnt || 0);
-			$tim = ($read_sec || 0) + ($write_sec || 0);
-
-			$str = $e . "_ioa" . $n;
-			$val = $ioa;
-			$ioa = $val - ($config->{fs_hist}->{$str} || 0);
-			$ioa = 0 unless $val != $ioa;
-			$ioa /= 60;
-			$config->{fs_hist}->{$str} = $val;
-
-			$str = $e . "_tim" . $n;
-			$val = $tim;
-			$tim = $val - ($config->{fs_hist}->{$str} || 0);
-			$tim = 0 unless $val != $tim;
-			$tim /= 60;
-			$config->{fs_hist}->{$str} = $val;
-
-			$rrdata .= ":$use:$ioa:$tim:$ino:0:0:0:0";
+		# restore the original values from <devmap>
+		foreach my $d (sort keys %{$config->{fs_orig_devmap}}) {
+			$config->{fs}->{devmap}->{$d} = $config->{fs_orig_devmap}->{$d};
 		}
-		$e++;
+		get_device_name($myself, $config, $fs, $debug);
 	}
 
+	my $rrdata = measure($myself, $config, $fs);
+
 	RRDs::update($rrd, $rrdata);
 	logger("$myself: $rrdata") if $debug;
 	my $err = RRDs::error;
@@ -862,7 +911,7 @@ sub fs_cgi {
 	}
 
 	$title = !$silent ? $title : "";
-
+	my $gap_on_all_nan = lc($fs->{gap_on_all_nan} || "") eq "y" ? 1 : 0;
 
 	# text mode
 	#
@@ -1008,6 +1057,7 @@ sub fs_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
+		my $cdef_allvalues_fs = $gap_on_all_nan ? "CDEF:allvalues=fs0,UN,0,1,IF,fs1,UN,0,1,IF,fs2,UN,0,1,IF,fs3,UN,0,1,IF,fs4,UN,0,1,IF,fs5,UN,0,1,IF,fs6,UN,0,1,IF,fs7,UN,0,1,IF,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4]",
 			"--title=$config->{graphs}->{_fs1}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -1028,7 +1078,7 @@ sub fs_cgi {
 			"DEF:fs5=$rrd:fs" . $e . "_use5:AVERAGE",
 			"DEF:fs6=$rrd:fs" . $e . "_use6:AVERAGE",
 			"DEF:fs7=$rrd:fs" . $e . "_use7:AVERAGE",
-			"CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+",
+			$cdef_allvalues_fs,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -1056,7 +1106,7 @@ sub fs_cgi {
 				"DEF:fs5=$rrd:fs" . $e . "_use5:AVERAGE",
 				"DEF:fs6=$rrd:fs" . $e . "_use6:AVERAGE",
 				"DEF:fs7=$rrd:fs" . $e . "_use7:AVERAGE",
-				"CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+",
+				$cdef_allvalues_fs,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -1127,6 +1177,7 @@ sub fs_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_ioa = $gap_on_all_nan ? "CDEF:allvalues=ioa0,UN,0,1,IF,ioa1,UN,0,1,IF,ioa2,UN,0,1,IF,ioa3,UN,0,1,IF,ioa4,UN,0,1,IF,ioa5,UN,0,1,IF,ioa6,UN,0,1,IF,ioa7,UN,0,1,IF,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=ioa0,ioa1,ioa2,ioa3,ioa4,ioa5,ioa6,ioa7,+,+,+,+,+,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 1]",
 			"--title=$config->{graphs}->{_fs2}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -1147,7 +1198,7 @@ sub fs_cgi {
 			"DEF:ioa5=$rrd:fs" . $e . "_ioa5:AVERAGE",
 			"DEF:ioa6=$rrd:fs" . $e . "_ioa6:AVERAGE",
 			"DEF:ioa7=$rrd:fs" . $e . "_ioa7:AVERAGE",
-			"CDEF:allvalues=ioa0,ioa1,ioa2,ioa3,ioa4,ioa5,ioa6,ioa7,+,+,+,+,+,+,+",
+			$cdef_allvalues_ioa,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -1175,7 +1226,7 @@ sub fs_cgi {
 				"DEF:ioa5=$rrd:fs" . $e . "_ioa5:AVERAGE",
 				"DEF:ioa6=$rrd:fs" . $e . "_ioa6:AVERAGE",
 				"DEF:ioa7=$rrd:fs" . $e . "_ioa7:AVERAGE",
-				"CDEF:allvalues=ioa0,ioa1,ioa2,ioa3,ioa4,ioa5,ioa6,ioa7,+,+,+,+,+,+,+",
+				$cdef_allvalues_ioa,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -1243,6 +1294,7 @@ sub fs_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
+		my $cdef_allvalues_ino = $gap_on_all_nan ? "CDEF:allvalues=fs0,UN,0,1,IF,fs1,UN,0,1,IF,fs2,UN,0,1,IF,fs3,UN,0,1,IF,fs4,UN,0,1,IF,fs5,UN,0,1,IF,fs6,UN,0,1,IF,fs7,UN,0,1,IF,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 2]",
 			"--title=$config->{graphs}->{_fs3}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -1263,7 +1315,7 @@ sub fs_cgi {
 			"DEF:fs5=$rrd:fs" . $e . "_ino5:AVERAGE",
 			"DEF:fs6=$rrd:fs" . $e . "_ino6:AVERAGE",
 			"DEF:fs7=$rrd:fs" . $e . "_ino7:AVERAGE",
-			"CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+",
+			$cdef_allvalues_ino,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -1291,7 +1343,7 @@ sub fs_cgi {
 				"DEF:fs5=$rrd:fs" . $e . "_ino5:AVERAGE",
 				"DEF:fs6=$rrd:fs" . $e . "_ino6:AVERAGE",
 				"DEF:fs7=$rrd:fs" . $e . "_ino7:AVERAGE",
-				"CDEF:allvalues=fs0,fs1,fs2,fs3,fs4,fs5,fs6,fs7,+,+,+,+,+,+,+",
+				$cdef_allvalues_ino,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -1397,6 +1449,7 @@ sub fs_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_tim = $gap_on_all_nan ? "CDEF:allvalues=tim0,UN,0,1,IF,tim1,UN,0,1,IF,tim2,UN,0,1,IF,tim3,UN,0,1,IF,tim4,UN,0,1,IF,tim5,UN,0,1,IF,tim6,UN,0,1,IF,tim7,UN,0,1,IF,+,+,+,+,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=tim0,tim1,tim2,tim3,tim4,tim5,tim6,tim7,+,+,+,+,+,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 3]",
 			"--title=$graph_title",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -1417,7 +1470,7 @@ sub fs_cgi {
 			"DEF:tim5=$rrd:fs" . $e . "_tim5:AVERAGE",
 			"DEF:tim6=$rrd:fs" . $e . "_tim6:AVERAGE",
 			"DEF:tim7=$rrd:fs" . $e . "_tim7:AVERAGE",
-			"CDEF:allvalues=tim0,tim1,tim2,tim3,tim4,tim5,tim6,tim7,+,+,+,+,+,+,+",
+			$cdef_allvalues_tim,
 			"CDEF:stim0=tim0,1000,/",
 			"CDEF:stim1=tim1,1000,/",
 			"CDEF:stim2=tim2,1000,/",
@@ -1453,7 +1506,7 @@ sub fs_cgi {
 				"DEF:tim5=$rrd:fs" . $e . "_tim5:AVERAGE",
 				"DEF:tim6=$rrd:fs" . $e . "_tim6:AVERAGE",
 				"DEF:tim7=$rrd:fs" . $e . "_tim7:AVERAGE",
-				"CDEF:allvalues=tim0,tim1,tim2,tim3,tim4,tim5,tim6,tim7,+,+,+,+,+,+,+",
+				$cdef_allvalues_tim,
 				"CDEF:stim0=tim0,1000,/",
 				"CDEF:stim1=tim1,1000,/",
 				"CDEF:stim2=tim2,1000,/",
