@@ -147,39 +147,48 @@ sub nut_update {
 	my $n;
 	my $rrdata = "N";
 
+	my $use_nan_for_missing_data = lc($nut->{use_nan_for_missing_data} || "") eq "y" ? 1 : 0;
+	my $ignore_error_output = lc($nut->{ignore_error_output} || "") eq "y" ? 1 : 0;
+
 	my $e = 0;
 	foreach my $ups (my @nl = split(',', $nut->{list})) {
-		my $ltran = 0;
-		my $htran = 0;
-		my $ivolt = 0;
-		my $ovolt = 0;
-		my $bchar = 0;
-		my $loadc = 0;
-		my $mbatc = 0;
-		my $nxfer = 0;
-		my $atemp = 0;
-		my $itemp = 0;
-		my $humid = 0;
-		my $battv = 0;
-		my $nomba = 0;
-		my $timel = 0;
-		my $minti = 0;
-		my $linef = 0;
-		my $val01 = 0;
-		my $val02 = 0;
-		my $val03 = 0;
-		my $val04 = 0;
-		my $val05 = 0;
+		my $default_value = $use_nan_for_missing_data ? (0+"nan") : 0;
+
+		my $ltran = $default_value;
+		my $htran = $default_value;
+		my $ivolt = $default_value;
+		my $ovolt = $default_value;
+		my $bchar = $default_value;
+		my $loadc = $default_value;
+		my $mbatc = $default_value;
+		my $nxfer = $default_value;
+		my $atemp = $default_value;
+		my $itemp = $default_value;
+		my $humid = $default_value;
+		my $battv = $default_value;
+		my $nomba = $default_value;
+		my $timel = $default_value;
+		my $minti = $default_value;
+		my $linef = $default_value;
+		my $val01 = $default_value;
+		my $val02 = $default_value;
+		my $val03 = $default_value;
+		my $val04 = $default_value;
+		my $val05 = $default_value;
 
 		my $data;
-		if(open(PIPE, "upsc $ups |")) {
+		my $upsc_cmd = "upsc $ups";
+		if ($ignore_error_output) {
+			$upsc_cmd .= " 2>/dev/null";
+		}
+		if(open(PIPE, "$upsc_cmd |")) {
 			while(<PIPE>) { $data .= $_; }
 			close(PIPE);
 		}
 
 		if(!$data) {
-			logger("$myself: unable to execute 'upsc $ups' command or invalid connection.");
-			$rrdata .= ":$ltran:$htran:$ivolt:$ovolt:$bchar:$loadc:$mbatc:$nxfer:$atemp:$itemp:$humid:$battv:$nomba:$timel:$minti:$linef:0:0:0:0:0";
+			logger("$myself: unable to execute '$upsc_cmd' command or invalid connection.");
+			$rrdata .= ":$ltran:$htran:$ivolt:$ovolt:$bchar:$loadc:$mbatc:$nxfer:$atemp:$itemp:$humid:$battv:$nomba:$timel:$minti:$linef:$default_value:$default_value:$default_value:$default_value:$default_value";
 			next;
 		}
 
@@ -241,6 +250,30 @@ sub nut_update {
 	logger("ERROR: while updating $rrd: $err") if $err;
 }
 
+sub skipscale_string {
+	my ($skipscale) = @_;
+	if ($skipscale) {
+		return ":skipscale";
+	} else {
+		return "";
+	}
+}
+
+sub altscaling_options {
+	my ($altscaling) = @_;
+	my @scaling_options;
+	if ($altscaling) {
+		push(@scaling_options, "--alt-autoscale");
+		push(@scaling_options, "--alt-y-grid");
+	}
+	return @scaling_options;
+}
+
+sub pad_string {
+	my ($string_length, $string) = @_;
+	return sprintf("%-" . $string_length . "s",$string);
+}
+
 sub nut_cgi {
 	my ($package, $config, $cgi) = @_;
 	my @output;
@@ -299,6 +332,14 @@ sub nut_cgi {
 		$temp_scale = "Fahrenheit";
 	}
 
+	my $gap_on_all_nan = lc($nut->{gap_on_all_nan} || "") eq "y" ? 1 : 0;
+	my $ignore_error_output = lc($nut->{ignore_error_output} || "") eq "y" ? 1 : 0;
+	my $skipscale_for_transfer_voltage = lc($nut->{skipscale_for_transfer_voltage} || "") eq "y" ? 1 : 0;
+	my $skipscale_for_shutdown_level = lc($nut->{skipscale_for_shutdown_level} || "") eq "y" ? 1 : 0;
+	my $alt_scaling_for_voltage = lc($nut->{alt_scaling_for_voltage} || "") eq "y" ? 1 : 0;
+	my $alt_scaling_for_timeleft = lc($nut->{alt_scaling_for_timeleft} || "") eq "y" ? 1 : 0;
+	my $alt_scaling_for_battery_voltage = lc($nut->{alt_scaling_for_battery_voltage} || "") eq "y" ? 1 : 0;
+
 	# text mode
 	#
 	if(lc($config->{iface_mode}) eq "text") {
@@ -388,7 +429,11 @@ sub nut_cgi {
 	foreach my $ups (my @nl = split(',', $nut->{list})) {
 
 		my $data;
-		if(open(PIPE, "upsc $ups |")) {
+		my $upsc_cmd = "upsc $ups";
+		if ($ignore_error_output) {
+			$upsc_cmd .= " 2>/dev/null";
+		}
+		if(open(PIPE, "$upsc_cmd |")) {
 			while(<PIPE>) { $data .= $_; }
 			close(PIPE);
 		}
@@ -398,7 +443,8 @@ sub nut_cgi {
 		my $driver = "";
 		my $model = "";
 		my $status = "";
-		my $transfer = "";
+		my $transfer = "None";
+		my $nominal_power;
 		foreach(my @l = split('\n', $data)) {
 			if(/^driver\.name:\s+(.*?)$/) {
 				$driver = trim($1);
@@ -420,6 +466,10 @@ sub nut_cgi {
 				$transfer = trim($1);
 				next;
 			}
+			if(/^ups\.realpower\.nominal:\s+(.*?)$/) {
+				$nominal_power = trim($1);
+				next;
+			}
 		}
 		if($RRDs::VERSION > 1.2) {
 			$driver = "COMMENT: $driver\\: $model ($status)\\c",
@@ -443,7 +493,7 @@ sub nut_cgi {
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
-		push(@tmp, "LINE2:htran#EE4444:High input transfer");
+		push(@tmp, "LINE2:htran#EE4444:High input transfer" . skipscale_string($skipscale_for_transfer_voltage));
 		push(@tmp, "GPRINT:htran:LAST:   Cur\\: %5.1lf");
 		push(@tmp, "GPRINT:htran:AVERAGE:  Avg\\: %5.1lf");
 		push(@tmp, "GPRINT:htran:MIN:  Min\\: %5.1lf");
@@ -458,15 +508,15 @@ sub nut_cgi {
 		push(@tmp, "GPRINT:ovolt:AVERAGE:  Avg\\: %5.1lf");
 		push(@tmp, "GPRINT:ovolt:MIN:  Min\\: %5.1lf");
 		push(@tmp, "GPRINT:ovolt:MAX:  Max\\: %5.1lf\\n");
-		push(@tmp, "LINE2:ltran#EE4444:Low input transfer");
+		push(@tmp, "LINE2:ltran#EE4444:Low input transfer" . skipscale_string($skipscale_for_transfer_voltage));
 		push(@tmp, "GPRINT:ltran:LAST:    Cur\\: %5.1lf");
 		push(@tmp, "GPRINT:ltran:AVERAGE:  Avg\\: %5.1lf");
 		push(@tmp, "GPRINT:ltran:MIN:  Min\\: %5.1lf");
 		push(@tmp, "GPRINT:ltran:MAX:  Max\\: %5.1lf\\n");
-		push(@tmpz, "LINE2:htran#EE4444:High input transfer");
+		push(@tmpz, "LINE2:htran#EE4444:High input transfer" . skipscale_string($skipscale_for_transfer_voltage));
 		push(@tmpz, "LINE2:ivolt#44EE44:Input voltage");
 		push(@tmpz, "LINE2:ovolt#4444EE:Output voltage");
-		push(@tmpz, "LINE2:ltran#EE4444:Low input transfer");
+		push(@tmpz, "LINE2:ltran#EE4444:Low input transfer" . skipscale_string($skipscale_for_transfer_voltage));
 		if(lc($config->{show_gaps}) eq "y") {
 			push(@tmp, "AREA:wrongdata#$colors->{gap}:");
 			push(@tmpz, "AREA:wrongdata#$colors->{gap}:");
@@ -479,6 +529,7 @@ sub nut_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
+		my $cdef_allvalues_volt = $gap_on_all_nan ? "CDEF:allvalues=ltran,UN,0,1,IF,htran,UN,0,1,IF,ivolt,UN,0,1,IF,ovolt,UN,0,1,IF,+,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=ltran,htran,ivolt,ovolt,+,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6]",
 			"--title=$config->{graphs}->{_nut1}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -486,6 +537,7 @@ sub nut_cgi {
 			"--vertical-label=Volts",
 			"--width=$width",
 			"--height=$height",
+			altscaling_options($alt_scaling_for_voltage),
 			@extra,
 			@riglim,
 			$zoom,
@@ -495,7 +547,7 @@ sub nut_cgi {
 			"DEF:htran=$rrd:nut" . $e . "_htran:AVERAGE",
 			"DEF:ivolt=$rrd:nut" . $e . "_ivolt:AVERAGE",
 			"DEF:ovolt=$rrd:nut" . $e . "_ovolt:AVERAGE",
-			"CDEF:allvalues=ltran,htran,ivolt,ovolt,+,+,+",
+			$cdef_allvalues_volt,
 			@CDEF,
 			"COMMENT: \\n",
 			@tmp,
@@ -513,6 +565,7 @@ sub nut_cgi {
 				"--width=$width",
 				"--height=$height",
 				@full_size_mode,
+				altscaling_options($alt_scaling_for_voltage),
 				@extra,
 				@riglim,
 				$zoom,
@@ -522,7 +575,7 @@ sub nut_cgi {
 				"DEF:htran=$rrd:nut" . $e . "_htran:AVERAGE",
 				"DEF:ivolt=$rrd:nut" . $e . "_ivolt:AVERAGE",
 				"DEF:ovolt=$rrd:nut" . $e . "_ovolt:AVERAGE",
-				"CDEF:allvalues=ltran,htran,ivolt,ovolt,+,+,+",
+				$cdef_allvalues_volt,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -548,29 +601,31 @@ sub nut_cgi {
 			}
 		}
 
+		my $nominal_power_string_length = 19;
+		my $nominal_power_string = (defined($nominal_power) ? " (max. " . $nominal_power . "W)" : "");
 		@riglim = @{setup_riglim($rigid[1], $limit[1])};
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
-		push(@tmp, "AREA:bchar#4444EE:Charge");
-		push(@tmp, "GPRINT:bchar:LAST:             Cur\\: %4.1lf%%");
-		push(@tmp, "GPRINT:bchar:AVERAGE:   Avg\\: %4.1lf%%");
-		push(@tmp, "GPRINT:bchar:MIN:   Min\\: %4.1lf%%");
-		push(@tmp, "GPRINT:bchar:MAX:   Max\\: %4.1lf%%\\n");
-		push(@tmp, "AREA:loadc#EE4444:Load capacity");
-		push(@tmp, "GPRINT:loadc:LAST:      Cur\\: %4.1lf%%");
-		push(@tmp, "GPRINT:loadc:AVERAGE:   Avg\\: %4.1lf%%");
-		push(@tmp, "GPRINT:loadc:MIN:   Min\\: %4.1lf%%");
-		push(@tmp, "GPRINT:loadc:MAX:   Max\\: %4.1lf%%\\n");
+		push(@tmp, "AREA:bchar#4444EE:" . pad_string($nominal_power_string_length, "Charge"));
+		push(@tmp, "GPRINT:bchar:LAST:Cur\\:%5.1lf%%");
+		push(@tmp, "GPRINT:bchar:AVERAGE:   Avg\\:%5.1lf%%");
+		push(@tmp, "GPRINT:bchar:MIN:   Min\\:%5.1lf%%");
+		push(@tmp, "GPRINT:bchar:MAX:   Max\\:%5.1lf%%\\n");
+		push(@tmp, "AREA:loadc#EE4444:" . pad_string($nominal_power_string_length, "Load" . $nominal_power_string));
+		push(@tmp, "GPRINT:loadc:LAST:Cur\\:%5.1lf%%");
+		push(@tmp, "GPRINT:loadc:AVERAGE:   Avg\\:%5.1lf%%");
+		push(@tmp, "GPRINT:loadc:MIN:   Min\\:%5.1lf%%");
+		push(@tmp, "GPRINT:loadc:MAX:   Max\\:%5.1lf%%\\n");
 		push(@tmp, "LINE1:bchar#0000EE");
 		push(@tmp, "LINE1:loadc#EE0000");
-		push(@tmp, "LINE2:mbatc#EEEE44:Shutdown level");
-		push(@tmp, "GPRINT:mbatc:LAST:     Cur\\: %4.1lf%%");
-		push(@tmp, "GPRINT:mbatc:AVERAGE:   Avg\\: %4.1lf%%");
-		push(@tmp, "GPRINT:mbatc:MIN:   Min\\: %4.1lf%%");
-		push(@tmp, "GPRINT:mbatc:MAX:   Max\\: %4.1lf%%\\n");
+		push(@tmp, "LINE2:mbatc#EEEE44:" . pad_string($nominal_power_string_length, "Shutdown level"));
+		push(@tmp, "GPRINT:mbatc:LAST:Cur\\:%5.1lf%%");
+		push(@tmp, "GPRINT:mbatc:AVERAGE:   Avg\\:%5.1lf%%");
+		push(@tmp, "GPRINT:mbatc:MIN:   Min\\:%5.1lf%%");
+		push(@tmp, "GPRINT:mbatc:MAX:   Max\\:%5.1lf%%\\n");
 		push(@tmpz, "AREA:bchar#4444EE:Charge");
-		push(@tmpz, "AREA:loadc#EE4444:Load");
+		push(@tmpz, "AREA:loadc#EE4444:Load" . $nominal_power_string);
 		push(@tmpz, "LINE2:mbatc#EEEE44:Shutdown level");
 		if(lc($config->{show_gaps}) eq "y") {
 			push(@tmp, "AREA:wrongdata#$colors->{gap}:");
@@ -584,6 +639,7 @@ sub nut_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
+		my $cdef_allvalues_bat = $gap_on_all_nan ? "CDEF:allvalues=bchar,UN,0,1,IF,mbatc,UN,0,1,IF,loadc,UN,0,1,IF,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=bchar,mbatc,loadc,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6 + 1]",
 			"--title=$config->{graphs}->{_nut2}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -599,7 +655,7 @@ sub nut_cgi {
 			"DEF:bchar=$rrd:nut" . $e . "_bchar:AVERAGE",
 			"DEF:loadc=$rrd:nut" . $e . "_loadc:AVERAGE",
 			"DEF:mbatc=$rrd:nut" . $e . "_mbatc:AVERAGE",
-			"CDEF:allvalues=bchar,mbatc,loadc,+,+",
+			$cdef_allvalues_bat,
 			@CDEF,
 			"COMMENT: \\n",
 			@tmp,
@@ -626,7 +682,7 @@ sub nut_cgi {
 				"DEF:bchar=$rrd:nut" . $e . "_bchar:AVERAGE",
 				"DEF:loadc=$rrd:nut" . $e . "_loadc:AVERAGE",
 				"DEF:mbatc=$rrd:nut" . $e . "_mbatc:AVERAGE",
-				"CDEF:allvalues=bchar,mbatc,loadc,+,+",
+				$cdef_allvalues_bat,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -689,6 +745,7 @@ sub nut_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_temp = $gap_on_all_nan ? "CDEF:allvalues=itemp,UN,0,1,IF,atemp,UN,0,1,IF,humid,UN,0,1,IF,+,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=itemp,atemp,humid,+,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6 + 2]",
 			"--title=$config->{graphs}->{_nut3}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -705,7 +762,7 @@ sub nut_cgi {
 			"DEF:itemp=$rrd:nut" . $e . "_itemp:AVERAGE",
 			"DEF:atemp=$rrd:nut" . $e . "_atemp:AVERAGE",
 			"DEF:humid=$rrd:nut" . $e . "_humid:AVERAGE",
-			"CDEF:allvalues=itemp,atemp,humid,+,+",
+			$cdef_allvalues_temp,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -729,7 +786,7 @@ sub nut_cgi {
 				"DEF:itemp=$rrd:nut" . $e . "_itemp:AVERAGE",
 				"DEF:atemp=$rrd:nut" . $e . "_atemp:AVERAGE",
 				"DEF:humid=$rrd:nut" . $e . "_humid:AVERAGE",
-				"CDEF:allvalues=itemp,atemp,humid,+,+",
+				$cdef_allvalues_temp,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -779,6 +836,7 @@ sub nut_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_batvolt = $gap_on_all_nan ? "CDEF:allvalues=battv,UN,0,1,IF,nomba,UN,0,1,IF,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=battv,nomba,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6 + 3]",
 			"--title=$config->{graphs}->{_nut4}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -786,6 +844,7 @@ sub nut_cgi {
 			"--vertical-label=Volts",
 			"--width=$width",
 			"--height=$height",
+			altscaling_options($alt_scaling_for_battery_voltage),
 			@extra,
 			@riglim,
 			$zoom,
@@ -794,7 +853,7 @@ sub nut_cgi {
 			@{$colors->{graph_colors}},
 			"DEF:battv=$rrd:nut" . $e . "_battv:AVERAGE",
 			"DEF:nomba=$rrd:nut" . $e . "_nomba:AVERAGE",
-			"CDEF:allvalues=battv,nomba,+",
+			$cdef_allvalues_batvolt,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -809,6 +868,7 @@ sub nut_cgi {
 				"--width=$width",
 				"--height=$height",
 				@full_size_mode,
+				altscaling_options($alt_scaling_for_battery_voltage),
 				@extra,
 				@riglim,
 				$zoom,
@@ -817,7 +877,7 @@ sub nut_cgi {
 				@{$colors->{graph_colors}},
 				"DEF:battv=$rrd:nut" . $e . "_battv:AVERAGE",
 				"DEF:nomba=$rrd:nut" . $e . "_nomba:AVERAGE",
-				"CDEF:allvalues=battv,nomba,+",
+				$cdef_allvalues_batvolt,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -849,10 +909,10 @@ sub nut_cgi {
 		undef(@CDEF);
 		push(@tmp, "LINE2:timel_min#44EEEE:Minutes left");
 		push(@tmp, "GPRINT:timel_min:LAST:         Current\\: %3.0lf\\n");
-		push(@tmp, "LINE2:minti_min#EEEE44:Shutdown level");
+		push(@tmp, "LINE2:minti_min#EEEE44:Shutdown level" . skipscale_string($skipscale_for_shutdown_level));
 		push(@tmp, "GPRINT:minti_min:LAST:       Current\\: %3.0lf\\n");
 		push(@tmpz, "LINE2:timel_min#44EEEE:Minutes left");
-		push(@tmpz, "LINE2:minti_min#EEEE44:Shutdown level");
+		push(@tmpz, "LINE2:minti_min#EEEE44:Shutdown level" . skipscale_string($skipscale_for_shutdown_level));
 		if(lc($config->{show_gaps}) eq "y") {
 			push(@tmp, "AREA:wrongdata#$colors->{gap}:");
 			push(@tmpz, "AREA:wrongdata#$colors->{gap}:");
@@ -867,6 +927,7 @@ sub nut_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_timeleft = $gap_on_all_nan ? "CDEF:allvalues=timel,UN,0,1,IF,minti,UN,0,1,IF,+,0,GT,1,UNKN,IF" : "CDEF:allvalues=timel,minti,+";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6 + 4]",
 			"--title=$config->{graphs}->{_nut5}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -874,6 +935,7 @@ sub nut_cgi {
 			"--vertical-label=Minutes",
 			"--width=$width",
 			"--height=$height",
+			altscaling_options($alt_scaling_for_timeleft),
 			@extra,
 			@riglim,
 			$zoom,
@@ -882,7 +944,7 @@ sub nut_cgi {
 			@{$colors->{graph_colors}},
 			"DEF:timel=$rrd:nut" . $e . "_timel:AVERAGE",
 			"DEF:minti=$rrd:nut" . $e . "_minti:AVERAGE",
-			"CDEF:allvalues=timel,minti,+",
+			$cdef_allvalues_timeleft,
 			"CDEF:timel_min=timel,60,/",
 			"CDEF:minti_min=minti,60,/",
 			@CDEF,
@@ -898,6 +960,7 @@ sub nut_cgi {
 				"--vertical-label=Minutes",
 				"--width=$width",
 				"--height=$height",
+				altscaling_options($alt_scaling_for_timeleft),
 				@full_size_mode,
 				@extra,
 				@riglim,
@@ -907,7 +970,7 @@ sub nut_cgi {
 				@{$colors->{graph_colors}},
 				"DEF:timel=$rrd:nut" . $e . "_timel:AVERAGE",
 				"DEF:minti=$rrd:nut" . $e . "_minti:AVERAGE",
-				"CDEF:allvalues=timel,minti,+",
+				$cdef_allvalues_timeleft,
 				"CDEF:timel_min=timel,60,/",
 				"CDEF:minti_min=minti,60,/",
 				@CDEF,
@@ -956,6 +1019,7 @@ sub nut_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
+		my $cdef_allvalues_freq = $gap_on_all_nan ? "CDEF:allvalues=linef,UN,0,1,IF,0,GT,1,UNKN,IF" : "CDEF:allvalues=linef";
 		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 6 + 5]",
 			"--title=$config->{graphs}->{_nut6}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
@@ -970,7 +1034,7 @@ sub nut_cgi {
 			@{$cgi->{version12_small}},
 			@{$colors->{graph_colors}},
 			"DEF:linef=$rrd:nut" . $e . "_linef:AVERAGE",
-			"CDEF:allvalues=linef",
+			$cdef_allvalues_freq,
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
@@ -992,7 +1056,7 @@ sub nut_cgi {
 				@{$cgi->{version12_small}},
 				@{$colors->{graph_colors}},
 				"DEF:linef=$rrd:nut" . $e . "_linef:AVERAGE",
-				"CDEF:allvalues=linef",
+				$cdef_allvalues_freq,
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
@@ -1023,11 +1087,10 @@ sub nut_cgi {
 			push(@output, "    </tr>\n");
 
 			push(@output, "    <tr>\n");
-			push(@output, "      <td bgcolor='$colors->{title_bg_color}' colspan='2'>\n");
-			push(@output, "       <font face='Verdana, sans-serif' color='$colors->{title_fg_color}'>\n");
+			push(@output, "      <td class='td-title' colspan='2'>\n");
 			push(@output, "       <font size='-1'>\n");
-			push(@output, "        <b style='{color: " . $colors->{title_fg_color} . "}'>&nbsp;&nbsp;" . trim($ups) . "</b>\n");
-			push(@output, "       </font></font>\n");
+			push(@output, "        <b>&nbsp;&nbsp;" . trim($ups) . "</b>\n");
+			push(@output, "       </font>\n");
 			push(@output, "      </td>\n");
 			push(@output, "    </tr>\n");
 			push(@output, main::graph_footer());
