@@ -185,13 +185,13 @@ sub nvidiagpu_update {
 			if($n < scalar(@gpu_group)) {
 				my $str = trim($gpu_group[$n]);
 
-        open(IN, "nvidia-smi --format=csv,noheader,nounits -i $str --query-gpu=clocks.current.graphics,clocks.current.memory,utilization.gpu,utilization.memory,temperature.gpu,temperature.memory,fan.speed,pstate,power.draw,power.limit,memory.used,memory.total |");
+				open(IN, "nvidia-smi --format=csv,noheader,nounits -i $str --query-gpu=clocks.current.graphics,clocks.current.memory,utilization.gpu,utilization.memory,temperature.gpu,temperature.memory,fan.speed,pstate,power.draw,power.limit,memory.used,memory.total |");
 				while(<IN>) {
 					my @tmp = split(',', $_);
 					if(scalar(@tmp) > 1) { # To catch missing devices
 						for(my $n_sensor = 0; $n_sensor < scalar(@tmp); $n_sensor += 1) {
 							my $val = trim($tmp[$n_sensor]);
-							if($val ne "N/A") {
+							if($val !~ /N\/A/) {
 								if(substr($val, 0, 1) eq "P") {
 									$val = substr($val, 1);
 								}
@@ -444,13 +444,13 @@ sub nvidiagpu_cgi {
 		"%4.2lf%s",
 		"%3.0lf%%",
 		"%3.0lf%%",
-		"%3.1lf",
-		"%3.1lf",
-		"%3.1lf%%",
+		"%5.1lf",
+		"%5.1lf",
+		"%5.1lf%%",
 		"%1.0lf",
 		"%5.0lf%s",
 		"%5.0lf%s",
-		"%3.1lf%%"
+		"%5.1lf%%"
 	);
 
 	my @graphs_per_plot = (6, 4, 5, 10, [8, 9], 2, 3, 0, 1, 7); # To rearange the graphs
@@ -555,12 +555,12 @@ sub nvidiagpu_cgi {
 
 					if($n_plot < $main_sensor_plots) {
 						if($main_plots_with_average[$n_plot]) {
-							push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\: " . $legend_labels_per_sensor[$n_sensor]);
-							push(@tmp, "GPRINT:trans_" . $value_name . ":AVERAGE:  Average\\: " . $legend_labels_per_sensor[$n_sensor]);
-							push(@tmp, "GPRINT:trans_" . $value_name . ":MIN:  Min\\: " . $legend_labels_per_sensor[$n_sensor]);
-							push(@tmp, "GPRINT:trans_" . $value_name . ":MAX:  Max\\: " . $legend_labels_per_sensor[$n_sensor] . "\\n");
+							push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\:" . $legend_labels_per_sensor[$n_sensor]);
+							push(@tmp, "GPRINT:trans_" . $value_name . ":AVERAGE: Average\\:" . $legend_labels_per_sensor[$n_sensor]);
+							push(@tmp, "GPRINT:trans_" . $value_name . ":MIN: Min\\:" . $legend_labels_per_sensor[$n_sensor]);
+							push(@tmp, "GPRINT:trans_" . $value_name . ":MAX: Max\\:" . $legend_labels_per_sensor[$n_sensor] . "\\n");
 						} else {
-							push(@tmp, "GPRINT:trans_" . $value_name . ":LAST: Current\\: " . $legend_labels_per_sensor[$n_sensor] . "\\n");
+							push(@tmp, "GPRINT:trans_" . $value_name . ":LAST:  Current\\: " . $legend_labels_per_sensor[$n_sensor] . "\\n");
 						}
 					} else {
 						if($show_current_values) {
@@ -632,7 +632,7 @@ sub nvidiagpu_cgi {
 				$cdef_sensor_allvalues .= ",0,GT,1,UNKN,IF";
 			}
 			my $plot_title = $config->{graphs}->{'_nvidiagpu' . ($n_plot + 1)};
-			$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 3 + $n_plot]",
+			$pic = $rrd{$version}->("$IMG_DIR" . $IMG[$e * $number_of_sensor_values_in_use + $n_plot],
 				"--title=$plot_title ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
 				"--imgformat=$imgfmt_uc",
@@ -650,10 +650,10 @@ sub nvidiagpu_cgi {
 				@CDEF,
 				@tmp);
 			$err = RRDs::error;
-			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 3 + $n_plot]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . $IMG[$e * $number_of_sensor_values_in_use + $n_plot] . ": $err\n") if $err;
 			if(lc($config->{enable_zoom}) eq "y") {
 				($width, $height) = split('x', $config->{graph_size}->{zoom});
-				$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 3 + $n_plot]",
+				$picz = $rrd{$version}->("$IMG_DIR" . $IMGz[$e * $number_of_sensor_values_in_use + $n_plot],
 					"--title=$plot_title  ($tf->{nwhen}$tf->{twhen})",
 					"--start=-$tf->{nwhen}$tf->{twhen}",
 					"--imgformat=$imgfmt_uc",
@@ -672,13 +672,13 @@ sub nvidiagpu_cgi {
 					@CDEF,
 					@tmpz);
 				$err = RRDs::error;
-				push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 3 + $n_plot]: $err\n") if $err;
+				push(@output, "ERROR: while graphing $IMG_DIR" . $IMGz[$e * $number_of_sensor_values_in_use + $n_plot]. ": $err\n") if $err;
 			}
 			$e2 = $e + $n_plot + 1;
 			if($title || ($silent =~ /imagetag/ && $graph =~ /nvidiagpu$e2/)) {
 				if(lc($config->{enable_zoom}) eq "y") {
 					if(lc($config->{disable_javascript_void}) eq "y") {
-						push(@output, "      " . picz_a_element(config => $config, IMGz => $IMGz[$e * 3 + $n_plot], IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+						push(@output, "      " . picz_a_element(config => $config, IMGz => $IMGz[$e * $number_of_sensor_values_in_use + $n_plot], IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 					} else {
 						if($version eq "new") {
 							$picz_width = $picz->{image_width} * $config->{global_zoom};
@@ -687,10 +687,10 @@ sub nvidiagpu_cgi {
 							$picz_width = $width + 115;
 							$picz_height = $height + 100;
 						}
-						push(@output, "      " . picz_js_a_element(width => $picz_width, height => $picz_height, config => $config, IMGz => $IMGz[$e * 3 + $n_plot], IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+						push(@output, "      " . picz_js_a_element(width => $picz_width, height => $picz_height, config => $config, IMGz => $IMGz[$e * $number_of_sensor_values_in_use + $n_plot], IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 					}
 				} else {
-					push(@output, "      " . img_element(config => $config, IMG => $IMG[$e * 3 + $n_plot]) . "\n");
+					push(@output, "      " . img_element(config => $config, IMG => $IMG[$e * $number_of_sensor_values_in_use + $n_plot]) . "\n");
 				}
 			}
 		}
@@ -701,11 +701,10 @@ sub nvidiagpu_cgi {
 
 			if($nvidiagpu->{desc}->{$k}) {
 				push(@output, "    <tr>\n");
-				push(@output, "      <td bgcolor='$colors->{title_bg_color}' colspan='2'>\n");
-				push(@output, "       <font face='Verdana, sans-serif' color='$colors->{title_fg_color}'>\n");
+				push(@output, "      <td class='td-title' colspan='2'>\n");
 				push(@output, "       <font size='-1'>\n");
-				push(@output, "        <b>&nbsp;&nbsp;$nvidiagpu->{desc}->{$k}<b>\n");
-				push(@output, "       </font></font>\n");
+				push(@output, "        <b>&nbsp;&nbsp;$nvidiagpu->{desc}->{$k}</b>\n");
+				push(@output, "       </font>\n");
 				push(@output, "      </td>\n");
 				push(@output, "    </tr>\n");
 			}
